// NOTE: We also need to compute V - overflow
// It should be ((A7 xor B7) xor Y7) xor C8 = or C7 xor C8
module ALU_module ( dralu_bar, A, B, func, M, carry_in, out, carry_out, AeqBbar, negative, overflow_bar, clk, reset_bar, ld_ie, iedata, ie_bar );

input dralu_bar;
input [7:0] A, B;
input [3:0] func;
input carry_in, M; 
input clk, reset_bar, ld_ie, iedata;
output [7:0] out;
output carry_out, AeqBbar, negative, overflow_bar;
output ie_bar;

wire [7:0] result;
wire carry_lo;
wire equal_lo;
wire equal_hi;
wire A7xorB7, A7xorB7xorY7;
wire nc;

wire x_lo;
wire y_lo;
wire x_hi;
wire y_hi;

// the chips
ALU_74181 u0(func, A[3:0], B[3:0], M, carry_in, result[3:0], x_lo, y_lo, carry_lo, equal_lo);
ALU_74181 u1(func, A[7:4], B[7:4], M, carry_lo, result[7:4], x_hi, y_hi, carry_out, equal_hi);
Buffer_74244 u2(dralu_bar, dralu_bar, result, out);
//QuadXOR_7486 u3(A[7], B[7], A7xorB7, A7xorB7, result[7], A7xorB7xorY7, A7xorB7xorY7, carry_out, overflow_bar, nc, nc, nc);
//Comparator_74LS688 u4( gnd, {gnd, gnd, gnd, gnd, gnd, gnd, gnd, gnd} , result, AeqBbar );

assign negative = result[7];
 
// overflow_bar and AeqBbar are computed by a 22v10 GAL
//assign overflow_bar = A[7] ^ B[7] ^ result[7] ^ carry_out;
assign overflow_bar = ~ ((~A[7] & ~B[7] & result[7] & ~func[1]) | (A[7] & B[7] & ~result[7] & ~func[1]) | (~A[7] & B[7] & result[7] & func[1]) | (A[7] & ~B[7] & ~result[7] & func[1]));
assign AeqBbar = (result[0] | result[1] | result[2] | result[3] | result[4] | result[5] | result[6] | result[7]);

reg ie_internal;
always @ (posedge clk)
begin 
  if (~reset_bar) begin
    ie_internal <= 1;
  end
  else if (~ld_ie) begin
	ie_internal <= iedata; 
  end
end
assign #15 ie_bar = ie_internal;

endmodule
