// all enable/drive signals are active low
// TODO: 4-bit condition code register, all inputs available in parallel
// control ROM address is: opcode(8) | phase(4) | aluflag
// control ROM output is:
// aluleft(2) | aluright(2) | load(3) | dralu  ||  aluop(4) | alumode | alucarry | enablecc | ld_shift_cc  ||  incmar | incpc | cntsp | spdir         | ldop | adrsel(2) | nc
// aluleft(2) | aluright(2) | load(3) | dralu  ||  aluop(4) | alumode | alucarry | s0       | s1           ||  special(2)     | cntsp | spdir/ie_data | ldop | adrsel(2) | load(1)
module Control_module ( clk, irq_bar, ie_bar, reset_bar, external_data_bus, aluflagv, aluflagz, aluflagn, aluflagc, aluflaga,
						en_left_y, en_left_a, en_left_x, en_left_x7, en_left_0, en_right_t, en_right_adrlo, en_right_adrhi, en_right_adrbank,
						aluop, alumode, alucarry, ld_y, ld_a, ld_x, ld_t, ld_arlo, ld_arhi, ld_pclo, ld_pchi, ld_mem,
						en_alu, en_cc, load_shiftbar_cc, incmar, incpc, cntsp, spdir, adrsel_pc, adrsel_ar, adrsel_sp, ld_ie, ld_splo, ld_sphi, ld_pcbank, ld_arbank, ld_spbank);

input clk, irq_bar, ie_bar, reset_bar, aluflagv, aluflagz, aluflagn, aluflagc, aluflaga;
input [7:0] external_data_bus;
output en_left_y, en_left_a, en_left_x, en_left_x7, en_left_0, en_right_t, en_right_adrlo, en_right_adrhi, en_right_adrbank; 
output [3:0] aluop;
output alumode, alucarry, ld_y, ld_a, ld_x, ld_t, ld_arlo, ld_arhi, ld_pclo, ld_pchi, ld_mem;
output en_alu, en_cc, load_shiftbar_cc, incmar, incpc, cntsp, spdir, adrsel_pc, adrsel_ar, adrsel_sp, ld_ie, ld_splo, ld_sphi;
output ld_pcbank, ld_arbank, ld_spbank;
  
wire [7:0] op_out;
wire ld_op;
wire ld_op_effective;
wire [3:0] phase_out;
wire nc_rco;
wire nc_mux;
wire [23:0] rom_out;
wire reset;
wire nc;
wire en_left_yorx7;

reg gnd;
reg vcc;

initial begin
  gnd = 0;
  vcc = 1;
 end
 
// the chips
QuadAND_7408 resetgate(ld_op, reset_bar, ld_op_effective, nc, nc, nc, nc, nc, nc, nc, nc, nc);
//Register_74377 op_reg(clk, ld_op_effective, external_data_bus, op_out);
Counter_74LS163A phase(clk, reset_bar, ld_op, vcc, vcc, {gnd,gnd,gnd,gnd}, phase_out, nc_rco );
//ROM8Kx24 controlrom({op_out, phase_out, aluflag}, gnd, gnd, rom_out );
ROM128K controlrom0({op_out, phase_out, aluflagv, aluflagz, aluflagn, aluflagc, aluflaga}, gnd, gnd, rom_out[7:0] );
ROM128K controlrom1({op_out, phase_out, aluflagv, aluflagz, aluflagn, aluflagc, aluflaga}, gnd, gnd, rom_out[15:8] );
ROM128K controlrom2({op_out, phase_out, aluflagv, aluflagz, aluflagn, aluflagc, aluflaga}, gnd, gnd, rom_out[23:16] );
DualDecoder_74LS139 u0( gnd, rom_out[22], rom_out[23], {en_left_a, en_left_yorx7, en_left_x, en_left_0}, gnd, rom_out[20], rom_out[21], {en_right_t, en_right_adrlo, en_right_adrhi, en_right_adrbank} ); // decodes ALUleft and ALUright
Decoder_74LS138 u1(gnd, reset, rom_out[0], rom_out[17], rom_out[18], rom_out[19], { nc, nc, ld_a, ld_x, ld_y, ld_t, ld_splo, ld_mem} );
Decoder_74LS138 u3(gnd, rom_out[0], reset_bar, rom_out[17], rom_out[18], rom_out[19], {ld_pcbank, ld_pchi, ld_pclo, ld_arbank, ld_arhi, ld_arlo, ld_spbank, ld_sphi} );
QuadNAND_7400 uinv( reset_bar, reset_bar, reset, nc, nc, nc, nc, nc, nc, nc, nc, nc );
DualDecoder_74LS139 u2( reset, rom_out[1], rom_out[2], {adrsel_pc, adrsel_ar, adrsel_sp, nc_mux}, reset, rom_out[6], rom_out[7], {nc_mux, incpc, incmar, ld_ie} );

// OP register is implemented in a 22V10 GAL
reg [7:0] op_internal;
always @ (posedge clk)
  if (~reset_bar | (~ld_op & (irq_bar | ie_bar))) begin
    op_internal <= external_data_bus;
  end
  else if (~ld_op) begin
	op_internal <= 0; 
  end

assign #15 op_out = op_internal;

// y and x7 share a control bit: use x7 with address register destinations, otherwise use Y
// NOTE: SPLO is not included here. ALU_A will not be loaded when loading SPLO.
assign en_left_x7 = en_left_yorx7 | rom_out[0];
assign en_left_y = en_left_yorx7 | ~rom_out[0];
 
assign en_alu = rom_out[16];

assign aluop = rom_out[15:12];
assign alumode = rom_out[11];
assign alucarry = rom_out[10];
assign en_cc = rom_out[9];
assign load_shiftbar_cc = rom_out[8];

//assign incmar = rom_out[7];
//assign incpc = rom_out[6];
assign cntsp = rom_out[5];
assign spdir = rom_out[4];
assign ld_op = rom_out[3];

 
endmodule
