// need to invert ld_mem for use in generation of bus driver enable
// TODO: ld_mem should be gated with the low half of the clock, to prevent spurious writes
// RAM is only selected during low portion (2nd half) of clock, does that fix it?
// or will this cause glitches, and we need another out-of-phase clock?
// Watch for random memory overwrites when clock speed goes up, because ld_mem hasn't settled correctly by 2nd half of clock
module Memory_module ( clk, adrbus_lo, adrbus_hi, adrbus_bank, ld_mem, en_alu, internalbus, externalbus );

input clk, ld_mem, en_alu;
input [7:0] adrbus_lo, adrbus_hi, adrbus_bank;
inout [7:0] internalbus, externalbus;

wire dr_membar;
wire nc;
wire driver_enablebar;
wire ramsel_bar;
wire ramsel;
wire hwsel_bar;
wire romsel_bar;

wire testreg_enablebar;
wire [7:0] testreg_output;
wire ld_testregbar;

reg gnd;
reg vcc;

initial begin
  gnd = 0;
  vcc = 1;
end
 
// the chips
QuadNOR_7402 u0(ld_mem, ld_mem, dr_membar, dr_membar, en_alu, driver_enablebar, adrbus_hi[7], adrbus_hi[6], ramsel_bar, ramsel_bar, ramsel_bar, ramsel);
EightInputNAND_7430 u1( ramsel_bar, adrbus_hi[5], adrbus_hi[4], adrbus_hi[3], adrbus_hi[2], adrbus_hi[1], adrbus_hi[0], vcc, hwsel_bar );
QuadNAND_7400 u2( ramsel_bar, hwsel_bar, romsel_bar, nc, nc, nc, nc, nc, nc, nc, nc, nc );
BidirectionalBusDriver_74245 u3( ld_mem, driver_enablebar, externalbus, internalbus );
ROM16K progrom( {adrbus_hi[5:0], adrbus_lo}, romsel_bar, dr_membar, externalbus );
SRAM128K ram( {adrbus_bank[0], adrbus_hi, adrbus_lo}, clk, ramsel, dr_membar, ld_mem, externalbus ); 
Decoder_74LS138 u6( hwsel_bar, gnd, vcc, adrbus_lo[5], adrbus_lo[6], adrbus_lo[7], { nc, nc, nc, nc, nc, nc, nc, testreg_enablebar} );

// TODO: need an OR gate here? 7432
assign ld_testregbar = testreg_enablebar | ld_mem;

Register_74377 testhw( clk, ld_testregbar, externalbus, testreg_output );
Buffer_74244 testbuf( testreg_enablebar, dr_membar, testreg_output, externalbus );

endmodule
