module Top_module(clk, reset_bar_in);

input clk;
input reset_bar_in;

wire [7:0] internal_data_bus;
wire [7:0] adrbus_bank;
wire [7:0] adrbus_lo;
wire [7:0] adrbus_hi;
wire ld_y, ld_x, ld_t, en_left_y, en_left_a, en_left_x, en_left_x7, en_left_0, en_right_t, en_right_adrlo, en_right_adrhi, en_right_adrbank;
wire [7:0] left_bus;
wire [7:0] right_bus;
wire en_alu;
wire [3:0] aluop;
wire alumode, alucarry;
wire [7:0] external_data_bus;
wire carry_out, AeqBbar, negative, overflow;
wire s0;
wire s1;
wire shifter_v;
wire shifter_z;
wire shifter_n;
wire shifter_c;
wire nc_ld;
wire nc_incdec;
wire nc_adrsel;
wire ld_arlo, ld_arhi, ld_pclo, ld_pchi, ld_pcbank, ld_arbank, ld_spbank, ld_mem;
wire adrsel_pc, adrsel_ar, adrsel_sp;
wire incmar, incpc, cntsp, spdir; // spdir = spdir/ie_data
wire adrsel_pc_effective;
wire nc;
wire ld_ie;
wire ie_bar;
wire ld_splo, ld_sphi;

reg gnd;
reg vcc;
reg irq_bar;

initial begin
  gnd = 0;
  vcc = 1;
  irq_bar = 1;
  
  $readmemh("roms/microcode_rom_0.rom", contmod.controlrom0.mem);
  $readmemh("roms/microcode_rom_1.rom", contmod.controlrom1.mem);
  $readmemh("roms/microcode_rom_2.rom", contmod.controlrom2.mem);
  $readmemh("roms/program_rom.rom", memmod.progrom.mem);
end
 
// some handy names, for viewing waveforms
wire [7:0] opcode;
assign opcode = contmod.op_out;
wire [3:0] phase;
assign phase = contmod.phase.q;
wire [23:0] pc;
assign pc = { pc2.q_internal, pc1.q_internal, pc0.q_internal };
wire [7:0] databus;
assign databus = internal_data_bus;
wire [7:0] extbus;
assign extbus = external_bus;
wire [7:0] a;
assign a = regmod.areg.q;
wire [7:0] x;
assign x = regmod.xreg.q;
wire [7:0] y;
assign y = regmod.yreg.q;
wire [7:0] t;
assign t = regmod.treg.q;
reg previous_carry;
wire rco_bar_ar0, rco_bar_ar1;
wire rco_bar_pc0, rco_bar_pc1;
wire rco_bar_sp0, rco_bar_sp1;

// reset must be clocked, so it only changes on a clock edge
Register_74377 resetgate2( clk, gnd, {reset_bar_in, nc, nc, nc, nc, nc, nc, nc}, {reset_bar_gated, nc, nc, nc, nc, nc, nc, nc} );

// the chips
Registers_module regmod( clk, internal_data_bus, adrbus_lo, adrbus_hi, adrbus_bank, ld_y, ld_a, ld_x, ld_t, en_left_y, en_left_a, en_left_x, en_left_x7, en_left_0, en_right_t, en_right_adrlo, en_right_adrhi, en_right_adrbank, left_bus, right_bus );
ALU_module alumod( en_alu, left_bus, right_bus, aluop, alumode, alucarry, internal_data_bus, carry_out, AeqBbar, negative, overflow, clk, reset_bar_gated, ld_ie, spdir, ie_bar );
// Note that CC serial in is A[3]
//Shifter_74166 shifter( clk, reset_bar_gated, en_cc, load_shiftbar_cc, regmod.areg.q[3], { gnd, gnd, gnd, gnd, overflow, AeqBbar, negative, carry_out}, shifter_q );
Shifter_74LS194A shifter( clk, reset_bar_gated, s0, s1, gnd, regmod.xreg.q[3], { overflow, AeqBbar, negative, carry_out}, { shifter_v, shifter_z, shifter_n, shifter_c } );
Control_module contmod( clk, irq_bar, ie_bar, reset_bar_gated, external_data_bus, shifter_v, shifter_z, shifter_n, shifter_c, previous_carry,
						en_left_y, en_left_a, en_left_x, en_left_x7, en_left_0, en_right_t, en_right_adrlo, en_right_adrhi, en_right_adrbank,
						aluop, alumode, alucarry, ld_y, ld_a, ld_x, ld_t, ld_arlo, ld_arhi, ld_pclo, ld_pchi, ld_mem,
						en_alu, s0, s1, incmar, incpc, cntsp, spdir, adrsel_pc, adrsel_ar, adrsel_sp, ld_ie, ld_splo, ld_sphi, ld_pcbank, ld_arbank, ld_spbank);
						
QuadAND_7408 resetgate(adrsel_pc, reset_bar_gated, adrsel_pc_effective, nc, nc, nc, nc, nc, nc, nc, nc, nc);
//Counter16bit_module pcmod( clk, reset_bar_gated, ld_pclo, ld_pchi, incpc, vcc, adrsel_pc_effective, internal_data_bus, adrbus_lo, adrbus_hi );
Register_PC pc0( clk, reset_bar_gated, ld_pclo, incpc, adrsel_pc, internal_data_bus, adrbus_lo, rco_bar_pc0 );
Register_PC pc1( clk, reset_bar_gated, ld_pchi, rco_bar_pc0, adrsel_pc, internal_data_bus, adrbus_hi, rco_bar_pc1 );
Register_PC pc2( clk, reset_bar_gated, ld_pcbank, rco_bar_pc1, adrsel_pc, internal_data_bus, adrbus_bank, nc );
//Counter16bit_module ar( clk, reset_bar_gated, ld_arlo, ld_arhi, incmar, vcc, adrsel_ar, internal_data_bus, adrbus_lo, adrbus_hi );
Register_AR ar0( clk, ld_arlo, incmar, adrsel_ar, internal_data_bus, adrbus_lo, rco_bar_ar0 );
Register_AR ar1( clk, ld_arhi, rco_bar_ar0, adrsel_ar, internal_data_bus, adrbus_hi, rco_bar_ar1 );
Register_AR ar2( clk, ld_arbank, rco_bar_ar1, adrsel_ar, internal_data_bus, adrbus_bank, nc );
//Counter16bit_module sp( clk, reset_bar_gated, ld_splo, ld_sphi, cntsp, spdir, adrsel_sp, internal_data_bus, adrbus_lo, adrbus_hi );
Register_SP sp0( clk, ld_splo, cntsp, spdir, adrsel_sp, internal_data_bus, adrbus_lo, rco_bar_sp0 );
Register_SP sp1( clk, ld_sphi, rco_bar_sp0, spdir, adrsel_sp, internal_data_bus, adrbus_hi, rco_bar_sp1 );
Register_SP sp2( clk, ld_spbank, rco_bar_sp1, spdir, adrsel_sp, internal_data_bus, adrbus_bank, nc );
Memory_module memmod( clk, adrbus_lo, adrbus_hi, adrbus_bank, ld_mem, en_alu, internal_data_bus, external_data_bus );

// use a 7474 and a 7451?
always @ (posedge clk) 
begin
  // use arlo or pclo load enable
  // note: This doesn't really need to be gated with /reset, only Verilog seems to need it
  // NOTE: SPLO is not included here. ALU_A will not be loaded when loading SPLO.
  #25 previous_carry = ~(ld_pclo & ld_pchi & ld_pcbank & ld_arlo & ld_arhi & ld_arbank & ld_sphi & ld_spbank) & carry_out | ld_pclo & ld_pchi & ld_pcbank & ld_arlo & ld_arhi & ld_arbank & ld_sphi & ld_spbank & previous_carry | ~reset_bar_in;
end
 
endmodule
